var init = (function (gvc1) {
    'use strict';

    function delay(milliseconds) {
        return new Promise((resolve) => {
            setTimeout(() => resolve(undefined), milliseconds);
        });
    }

    const Main = imports.ui.main;
    const QuickSettings = Main.panel.statusArea.quickSettings;
    class AudioPanel {
        getDisplayedDevices() {
            const devices = QuickSettings._volume._output._deviceItems;
            return Array.from(devices, ([id, value]) => ({
                id,
                displayName: value.label.get_text(),
            }));
        }
        removeDevice(id) {
            QuickSettings._volume._output._removeDevice(id);
        }
        addDevice(id) {
            QuickSettings._volume._output._addDevice(id);
        }
    }

    function range(amount) {
        return [...Array(amount).keys()];
    }

    /**
     * Display name format copied from
     * https://gitlab.gnome.org/GNOME/gnome-shell/-/blob/main/js/ui/status/volume.js#L132
     * The "-" is U+2013 on purpose
     * @param device AudioDevice
     */
    function getAudioDevice(id, description, origin) {
        if (!description)
            description = "unknown description";
        if (!origin)
            origin = "unknown origin";
        return {
            id,
            displayName: `${description} – ${origin}`,
        };
    }

    class MixerWrapper {
        constructor(mixer, disposal) {
            this.mixer = mixer;
            this.disposal = disposal;
        }
        getAudioDevicesFromIds(ids) {
            return ids.map((id) => {
                const lookup = this.mixer.lookup_output_id(id);
                return getAudioDevice(id, lookup?.get_description(), lookup?.get_origin());
            });
        }
        /**
         * Uses a Dummy Device "trick" from
         * https://github.com/kgshank/gse-sound-output-device-chooser/blob/master/sound-output-device-chooser@kgshank.net/base.js#LL299C20-L299C20
         * @param displayNames display names
         * @returns A list of matching audio devices. If a given display name is not found,
         * undefined is returned in its place.
         */
        getAudioDevicesFromDisplayNames(displayNames) {
            const dummyDevice = new gvc1.MixerUIDevice();
            const devices = this.getAudioDevicesFromIds(range(dummyDevice.get_id()));
            return displayNames.map((name) => devices.find((device) => device.displayName === name));
        }
        subscribeToOutputChanges(callback) {
            const addId = this.mixer.connect("output-added", (_, deviceId) => callback({ deviceId, type: "output-added" }));
            const removeId = this.mixer.connect("output-removed", (_, deviceId) => callback({ deviceId, type: "output-removed" }));
            return { ids: [addId, removeId] };
        }
        unsubscribe(subscription) {
            subscription.ids.forEach((id) => {
                this.mixer.disconnect(id);
            });
        }
        dispose() {
            this.disposal();
        }
    }

    async function waitForMixerToBeReady(mixer) {
        while (mixer.get_state() === gvc1.MixerControlState.CONNECTING) {
            await delay(200);
        }
        const state = mixer.get_state();
        if (state === gvc1.MixerControlState.FAILED) {
            throw new Error("MixerControl is in a failed state");
        }
        else if (state === gvc1.MixerControlState.CLOSED) {
            throw new Error("MixerControl is in a closed state");
        }
    }

    class AudioPanelMixerSource {
        async getMixer() {
            const Volume = imports.ui.status.volume;
            const mixer = Volume.getMixerControl();
            await waitForMixerToBeReady(mixer);
            await delay(200);
            return new MixerWrapper(mixer, () => { });
        }
    }

    const ExtensionUtils$1 = imports.misc.extensionUtils;
    const SettingsPath = "org.gnome.shell.extensions.quicksettings-audio-devices-hider";
    const ExcludedOutputNamesSetting = "excluded-output-names";
    const AvailableOutputNames = "available-output-names";
    class SettingsUtils {
        static getExcludedOutputDeviceNames() {
            const settings = ExtensionUtils$1.getSettings(SettingsPath);
            const ids = settings.get_strv(ExcludedOutputNamesSetting);
            return ids;
        }
        static setExcludedOutputDeviceNames(displayNames) {
            const settings = ExtensionUtils$1.getSettings(SettingsPath);
            settings.set_strv(ExcludedOutputNamesSetting, displayNames);
        }
        static addToExcludedOutputDeviceNames(displayName) {
            const currentOutputs = SettingsUtils.getExcludedOutputDeviceNames();
            if (currentOutputs.includes(displayName)) {
                return;
            }
            const newOutputs = [...currentOutputs, displayName];
            const settings = ExtensionUtils$1.getSettings(SettingsPath);
            log("Setting " + newOutputs);
            settings.set_strv(ExcludedOutputNamesSetting, newOutputs);
        }
        static removeFromExcludedOutputDeviceNames(displayName) {
            const outputs = SettingsUtils.getExcludedOutputDeviceNames();
            const index = outputs.indexOf(displayName);
            if (index === -1) {
                return;
            }
            outputs.splice(index, 1);
            const settings = ExtensionUtils$1.getSettings(SettingsPath);
            settings.set_strv(ExcludedOutputNamesSetting, outputs);
        }
        static getAvailableOutputs() {
            const settings = ExtensionUtils$1.getSettings(SettingsPath);
            const ids = settings.get_strv(AvailableOutputNames);
            return ids;
        }
        static setAvailableOutputs(displayNames) {
            const settings = ExtensionUtils$1.getSettings(SettingsPath);
            settings.set_strv(AvailableOutputNames, displayNames.map((id) => id.toString()));
        }
        static addToAvailableOutputs(displayName) {
            const currentOutputs = SettingsUtils.getAvailableOutputs();
            if (currentOutputs.includes(displayName)) {
                return;
            }
            const newAllOutputs = [...currentOutputs, displayName];
            const settings = ExtensionUtils$1.getSettings(SettingsPath);
            settings.set_strv(AvailableOutputNames, newAllOutputs.map((id) => id.toString()));
        }
        static removeFromAvailableOutputs(displayName) {
            const outputs = SettingsUtils.getAvailableOutputs();
            const index = outputs.indexOf(displayName);
            if (index === -1) {
                return;
            }
            outputs.splice(index, 1);
            const settings = ExtensionUtils$1.getSettings(SettingsPath);
            settings.set_strv(AvailableOutputNames, outputs.map((id) => id.toString()));
        }
    }

    const ExtensionUtils = imports.misc.extensionUtils;
    class Extension {
        constructor(uuid) {
            this._mixer = null;
            this._mixerSubscription = null;
            this._uuid = uuid;
            this._audioPanel = new AudioPanel();
            this._settings = ExtensionUtils.getSettings(SettingsPath);
            this._lastExludedDevices = SettingsUtils.getExcludedOutputDeviceNames();
        }
        async enable() {
            log(`Enabling extension ${this._uuid}`);
            this._mixer = await new AudioPanelMixerSource().getMixer();
            this.setAllOutputsInSettings();
            this.setupOutputChangesSubscription();
            this.hideExcludedDevices();
            this.setupExludedDevicesHandling();
        }
        hideExcludedDevices() {
            var devices = this._mixer.getAudioDevicesFromDisplayNames(this._lastExludedDevices);
            devices.forEach((device) => {
                if (device) {
                    this._audioPanel.removeDevice(device.id);
                }
            });
        }
        setupExludedDevicesHandling() {
            this._settingsSubscription = this._settings.connect(`changed::${ExcludedOutputNamesSetting}`, () => {
                const newExcludedDevices = SettingsUtils.getExcludedOutputDeviceNames();
                const devicesToShowIds = this.getDeviceIdsToShow(newExcludedDevices);
                const devicesToHideIds = this.getDeviceIdsToHide(newExcludedDevices);
                devicesToShowIds.forEach((id) => this._audioPanel.addDevice(id));
                devicesToHideIds.forEach((id) => this._audioPanel.removeDevice(id));
                this._lastExludedDevices = newExcludedDevices;
            });
        }
        getDeviceIdsToHide(newExcludedDevices) {
            const devicesToHide = newExcludedDevices.filter((current) => !this._lastExludedDevices.includes(current));
            const devicesToHideIds = this._mixer.getAudioDevicesFromDisplayNames(devicesToHide)
                .filter((n) => n)
                .map((n) => n.id);
            return devicesToHideIds;
        }
        getDeviceIdsToShow(newExcludedDevices) {
            const devicesToShow = this._lastExludedDevices.filter((last) => !newExcludedDevices.includes(last));
            const devicesToShowIds = this._mixer.getAudioDevicesFromDisplayNames(devicesToShow)
                .filter((n) => n)
                .map((n) => n.id);
            return devicesToShowIds;
        }
        setupOutputChangesSubscription() {
            this._mixerSubscription = this._mixer.subscribeToOutputChanges((event) => {
                this.updateAvailableOutputsInSettings(event);
                if (event.type === "output-added") {
                    this.hideDeviceIfExcluded(event.deviceId);
                }
            });
        }
        hideDeviceIfExcluded(deviceId) {
            const deviceName = this._mixer.getAudioDevicesFromIds([deviceId])[0]
                .displayName;
            const excludedOutputs = SettingsUtils.getExcludedOutputDeviceNames();
            if (excludedOutputs.includes(deviceName)) {
                delay(200).then(() => {
                    // delay due to potential race condition with Quick Setting panel's code
                    this._audioPanel.removeDevice(deviceId);
                });
            }
        }
        setAllOutputsInSettings() {
            const allDisplayedDevices = this._audioPanel.getDisplayedDevices();
            SettingsUtils.setAvailableOutputs(allDisplayedDevices.map((i) => i.displayName));
        }
        updateAvailableOutputsInSettings(event) {
            const displayName = this._mixer.getAudioDevicesFromIds([event.deviceId])[0]
                .displayName;
            if (event.type === "output-added") {
                SettingsUtils.addToAvailableOutputs(displayName);
            }
            else if (event.type === "output-removed") {
                SettingsUtils.removeFromAvailableOutputs(displayName);
            }
            else {
                log(`WARN: Received an unsupported MixerEvent: ${event.type}`);
            }
        }
        disable() {
            log(`Disabling extension ${this._uuid}`);
            if (this._mixerSubscription) {
                this._mixer.unsubscribe(this._mixerSubscription);
            }
            if (this._mixer) {
                this._mixer.dispose();
            }
            this._settings.disconnect(this._settingsSubscription);
            this.enableAllDevices();
        }
        enableAllDevices() {
            const allDevices = SettingsUtils.getAvailableOutputs();
            const devicesToShowIds = this._mixer.getAudioDevicesFromDisplayNames(allDevices)
                .filter((n) => n)
                .map((n) => n.id);
            devicesToShowIds.forEach((id) => this._audioPanel.addDevice(id));
        }
    }
    function extension (meta) {
        return new Extension(meta.uuid);
    }

    return extension;

})(imports.gi.Gvc);
